"""
This file is auto generated. Do not edit this file manually.
Source code: [monorepo]/contracts/generator.py
Last generated: 2024-01-10 16:15:36.265198
"""

import typing
from dataclasses import dataclass

CloudApiAuthInfoAuthorId = str

CloudApiAuthInfoProjectId = str


@dataclass
class CloudApiAuthInfo:
    authorId: CloudApiAuthInfoAuthorId
    projectId: CloudApiAuthInfoProjectId


CommonUserId = str

CommonUserEmail = str

CommonUserRolesItemValues = ["workflow_viewer"]

CommonUserRolesItem = typing.Literal["workflow_viewer"]

CommonUserRoles = typing.List[CommonUserRolesItem]

CommonUserProjectId = str

CommonUserCreatedAt = str


@dataclass
class CommonUser:
    id: CommonUserId
    email: CommonUserEmail
    roles: CommonUserRoles
    projectId: CommonUserProjectId
    createdAt: CommonUserCreatedAt


CloudApiCliUserGetResponse = CommonUser

CloudApiCliBuildCreateResponseUrl = str

CloudApiCliBuildCreateResponseBuildId = str


@dataclass
class CloudApiCliBuildCreateResponse:
    url: CloudApiCliBuildCreateResponseUrl
    buildId: CloudApiCliBuildCreateResponseBuildId


CloudApiConsoleUserUpdateRequestRoles = CommonUserRoles


@dataclass
class CloudApiConsoleUserUpdateRequest:
    roles: CloudApiConsoleUserUpdateRequestRoles


CloudApiConsoleUserUpdateResponse = CommonUser

CloudApiConsoleUserListQueryOffset = int

CloudApiConsoleUserListQueryLimit = int


@dataclass
class CloudApiConsoleUserListQuery:
    offset: CloudApiConsoleUserListQueryOffset
    limit: CloudApiConsoleUserListQueryLimit


CloudApiConsoleUserListResponseItem = CommonUser

CloudApiConsoleUserListResponse = typing.List[CloudApiConsoleUserListResponseItem]

CloudApiConsoleUserCreateRequestEmail = CommonUserEmail

CloudApiConsoleUserCreateRequestRoles = CommonUserRoles


@dataclass
class CloudApiConsoleUserCreateRequest:
    email: CloudApiConsoleUserCreateRequestEmail
    roles: CloudApiConsoleUserCreateRequestRoles


CloudApiConsoleUserCreateResponse = CommonUser

CommonProjectId = str

CommonProjectName = str

CommonProjectSubdomain = str

CommonProjectOrganizationId = str

CommonProjectCustomDomain0 = type(None)

CommonProjectCustomDomain1 = str

CommonProjectCustomDomain = typing.Union[
    CommonProjectCustomDomain0, CommonProjectCustomDomain1
]


@dataclass
class CommonProject:
    id: CommonProjectId
    name: CommonProjectName
    subdomain: CommonProjectSubdomain
    organizationId: CommonProjectOrganizationId
    customDomain: CommonProjectCustomDomain


CloudApiConsoleProjectDuplicateResponse = CommonProject



from typing import Dict, Tuple

from .execution import RequestData
from .static_execution import StaticExecution


class HookExecution(StaticExecution):
    def handle_started(self):
        self.log("started", {"request": self.context["request"]})
        return super().handle_started()

    def handle_success(self) -> str:
        self.context["response"] = self.context.get("response", ("", 200, {}))
        self.log(
            "success",
            {
                "response": self.context["response"],
            },
        )
        return super().handle_success()

    def handle_failure(self, e: Exception) -> str:
        self.context["response"] = self.context.get("response", ("", 500, {}))
        self.log(
            "failed",
            {
                "response": self.context["response"],
                "error": str(e),
            },
        )
        return super().handle_failure(e)

    def handle_lock_failed(self) -> None:
        self.context["response"] = self.context.get("response", ("", 409, {}))
        return super().handle_lock_failed()

    def get_response(self) -> Tuple[Dict, int, Dict]:
        if "response" not in self.context:
            raise Exception("No response found")

        return self.context["response"]

    def setup_context(self, request: RequestData):
        self.context["request"] = (
            request.body,
            request.query_params,
            request.headers,
        )

        self.init_stage_run(request.query_params.get(self.abstra_run_key))



import flask

from ...utils import is_valid_email
from ...jwt_auth import endcode_fake_jwt
from ...utils.environment import IS_PREVIEW, AUTHN_URL, CLIENT_ID, PROJECT_ID


def get_fake_auth_proviver_bp():
    bp = flask.Blueprint("fake-provider", __name__)
    emails = {}

    @bp.post("/authenticate")
    def _authenticate():
        data = flask.request.get_json(force=True)
        if not data:
            return flask.abort(400)

        email = data.get("email")
        if not email or not is_valid_email(email):
            return flask.abort(400)

        emails[email] = True
        return {"ok": True}

    @bp.post("/verify")
    def _verify():
        data = flask.request.get_json(force=True)
        if not data:
            return flask.abort(400)

        email = data.get("email")
        token = data.get("token")
        if not email or not token:
            return flask.abort(400)

        if token == "000000":
            # for testing purposes
            return flask.abort(404)

        verified = emails.pop(email, False)
        if not verified:
            return flask.abort(404)

        jwt = endcode_fake_jwt(email)
        return {"jwt": jwt}

    return bp


def get_player_bp():
    bp = flask.Blueprint("auth", __name__)

    if not IS_PREVIEW and AUTHN_URL:
        provider = "abstra"
        authority = AUTHN_URL
    else:
        provider = "local"
        authority = f"/_auth"
        auth_provider_bp = get_fake_auth_proviver_bp()
        bp.register_blueprint(auth_provider_bp)

    @bp.get("/config")
    def _get_config():
        return {
            "authority": authority,
            "provider": provider,
            "clientId": CLIENT_ID,
            "projectId": PROJECT_ID,
        }

    return bp



from .CnpjInput import CnpjInput
from .NpsInput import NpsInput
from .DateInput import DateInput
from .MultipleChoiceInput import MultipleChoiceInput
from .CameraInput import CameraInput
from .MarkdownOutput import MarkdownOutput
from .EmailInput import EmailInput
from .TextareaInput import TextareaInput
from .RichTextInput import RichTextInput
from .NumberSliderInput import NumberSliderInput
from .HtmlOutput import HtmlOutput
from .CodeInput import CodeInput
from .TimeInput import TimeInput
from .ChecklistInput import ChecklistInput
from .PandasOutput import PandasOutput
from .FileInput import FileInput
from .TagInput import TagInput
from .RatingInput import RatingInput
from .CustomInput import CustomInput
from .FileOutput import FileOutput
from .NumberInput import NumberInput
from .ImageInput import ImageInput
from .CpfInput import CpfInput
from .PasswordInput import PasswordInput
from .LatexOutput import LatexOutput
from .IframeOutput import IframeOutput
from .CheckboxInput import CheckboxInput
from .VideoInput import VideoInput
from .CardsInput import CardsInput
from .TextOutput import TextOutput
from .CurrencyInput import CurrencyInput
from .PlotlyOutput import PlotlyOutput
from .ImageOutput import ImageOutput
from .PhoneInput import PhoneInput
from .TextInput import TextInput
from .LinkOutput import LinkOutput
from .PandasRowSelectionInput import PandasRowSelectionInput
from .ProgressOutput import ProgressOutput
from .ListInput import ListInput
from .DropdownInput import DropdownInput
from .ToggleInput import ToggleInput



from typing import List
from ...repositories.requirements import RequirementsRepository
from pkg_resources import get_distribution
from ..linter import LinterRule, LinterFix, LinterIssue


class AddAbstraToRequirements(LinterFix):
    label = "Add abstra to requirements.txt"

    def fix(self):
        requirements = RequirementsRepository.load()
        requirements.add("abstra", get_distribution("abstra").version)
        RequirementsRepository.save(requirements)


class AbstraNotInRequirementsFound(LinterIssue):
    def __init__(self) -> None:
        self.label = "Abstra is not in your requirements.txt file."
        self.fixes = [AddAbstraToRequirements()]


class MissingAbstraInRequirements(LinterRule):
    label = "Abstra should be in your requirements.txt"
    type = "bug"

    def find_issues(self) -> List[LinterIssue]:
        requirements = RequirementsRepository.load()
        if not requirements.has("abstra", get_distribution("abstra").version):
            return [AbstraNotInRequirementsFound()]
        else:
            return []



from typing import Type, List, Dict
from .strategies import condition_strategy, action_strategy, iterator_strategy
from ...repositories.stage_run import get_stage_run_repository, StageRunRepository
from ...execution.execution import Execution, UnsetStageRun
from ...execution.script_execution import ScriptExecution
from ...execution.job_execution import JobExecution
from ...repositories.stage_run import StageRun
from ...threaded import threaded
from ...repositories.project.project import (
    ProjectRepository,
    ConditionStage,
    IteratorStage,
    ControlStage,
    ScriptStage,
    JobStage,
)


class WorkflowEngine:
    def __init__(self, stage_run_repo: Type[StageRunRepository]):
        self.stage_run_repo = stage_run_repo

    def get_stage(self, id: str):
        # TODO: inject this
        project = ProjectRepository.load()
        return project.get_stage(id)

    def notify_ran(self, execution: Execution):
        if not execution.stage_run:
            raise UnsetStageRun()

        stage = execution.stage
        status = execution.status
        stage_run = execution.stage_run

        transition_type = f"{stage.type_name}s:{status}"
        next_stage_run_dtos = action_strategy(stage, stage_run, transition_type)
        self.pub(stage_run, next_stage_run_dtos)

    def pub(self, parent_stage_run: StageRun, stage_run_dtos: List[Dict]):
        for stage_run in self.stage_run_repo.create_next(
            parent_stage_run, stage_run_dtos
        ):
            self.consume(stage_run)

    @threaded
    def run_job(self, stage: JobStage):
        execution = JobExecution(stage)
        execution.run()
        self.notify_ran(execution)

    @threaded
    def run_script(self, stage: ScriptStage, stage_run: StageRun):
        execution = ScriptExecution(stage, stage_run.id)
        execution.run()
        self.notify_ran(execution)

    @threaded
    def run_control(self, stage: ControlStage, stage_run: StageRun):
        next_stage_run_dtos: List[Dict] = []
        self.stage_run_repo.change_state(stage_run.id, "running")

        if isinstance(stage, ConditionStage):
            next_stage_run_dtos = condition_strategy(stage, stage_run)

        if isinstance(stage, IteratorStage):
            next_stage_run_dtos = iterator_strategy(stage, stage_run)

        self.stage_run_repo.change_state(stage_run.id, "finished")
        self.pub(stage_run, next_stage_run_dtos)

    def consume(self, stage_run: StageRun):
        stage = self.get_stage(stage_run.stage)
        if not stage:
            raise Exception(f"Stage {stage_run.stage} not found")

        if isinstance(stage, ScriptStage):
            return self.run_script(stage, stage_run)

        if isinstance(stage, (ConditionStage, IteratorStage)):
            return self.run_control(stage, stage_run)


def factory():
    return WorkflowEngine(get_stage_run_repository())


workflow_engine = factory()



from unittest import TestCase
from .proxy import ReadOnlyProxyDict


def getter():
    return {"a": 1, "b": 2}


class ReadOnlyProxyDictTest(TestCase):
    def test_data(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertEqual(proxy.data, {"a": 1, "b": 2})

    def test_len(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertEqual(len(proxy), 2)

    def test_getitem(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertEqual(proxy["a"], 1)

    def test_iter(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertEqual(list(iter(proxy)), ["a", "b"])

    def test_contains(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertTrue("a" in proxy)
        self.assertFalse("c" in proxy)

    def test_str(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertEqual(str(proxy), "{'a': 1, 'b': 2}")

    def test_repr(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertEqual(repr(proxy), "{'a': 1, 'b': 2}")

    def test_get(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertEqual(proxy.get("a"), 1)
        self.assertEqual(proxy.get("c"), None)
        self.assertEqual(proxy.get("c", 3), 3)

    def test_copy(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertEqual(proxy.copy(), {"a": 1, "b": 2})

    def test_keys(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertEqual(list(proxy.keys()), ["a", "b"])

    def test_values(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertEqual(list(proxy.values()), [1, 2])

    def test_items(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertEqual(list(proxy.items()), [("a", 1), ("b", 2)])

    def test_has_key(self):
        proxy = ReadOnlyProxyDict(getter)

        self.assertTrue(proxy.has_key("a"))
        self.assertFalse(proxy.has_key("c"))



from abstra_internals.interface.sdk.forms.generated.inputs import *
from abstra_internals.interface.sdk.forms.generated.outputs import *
from abstra_internals.interface.sdk.forms.page import Page
from abstra_internals.interface.sdk.forms.step import run_steps
from abstra_internals.interface.sdk.forms import url_params
from abstra_internals.interface.sdk.forms.list_item_schema import (
    ListItemSchema,
)
from abstra_internals.interface.sdk.common import (
    execute_js,
    redirect,
    get_user,
    get_query_params,
    alert,
)



import flask, flask_sock

from ...settings import Settings
from ..utils import send_from_dist
from ..controller.main import MainController
from ..workflow_engine import workflow_engine
from ...execution.execution import RequestData
from ..controller import auth as auth_controller
from ...execution import HookExecution, FormExecution
from ...utils.environment import BUILD_ID, SIDECAR_SHARED_TOKEN, SHOW_WATERMARK


def get_player_bp(controller: MainController):
    bp = flask.Blueprint("player", __name__)
    sock = flask_sock.Sock(bp)

    auth_bp = auth_controller.get_player_bp()
    bp.register_blueprint(auth_bp, url_prefix="/_auth")

    @bp.get("/_workspace")
    def _get_workspace():
        workspace = controller.get_workspace()
        return workspace.browser_runner_dto

    @bp.get("/_pages/<string:path>")
    def _get_page(path):
        page_runtime = controller.get_form_by_path(path)
        if not page_runtime:
            flask.abort(404)

        return {
            page_runtime.type_name: {
                **page_runtime.browser_runner_dto,
                "workspace": controller.get_workspace().browser_runner_dto,
            }
        }

    @bp.get("/_version")
    def _get_version():
        return BUILD_ID

    @bp.get("/_settings")
    def _get_settings():
        return flask.jsonify({"show_watermark": SHOW_WATERMARK})

    @sock.route("/_socket")
    def _websocket(conn: flask_sock.Server):
        request_data = RequestData(
            query_params=flask.request.args,
            body=flask.request.get_data(as_text=True),
            headers=flask.request.headers,
            method=flask.request.method,
        )

        try:
            id = flask.request.args.get("id")
            if id is None:
                return conn.close(reason=400, message="No path")

            form = controller.get_form(id)
            if not form:
                return conn.close(reason=404, message="Not found")

            execution = FormExecution(
                is_initial=controller.is_initial(form.id),
                request=request_data,
                connection=conn,
                stage=form,
            )

            execution.run()
            workflow_engine.notify_ran(execution)
        finally:
            conn.close(message="Done")

    @bp.put("/_files")
    def _upload_file():
        files = flask.request.files
        filename = flask.request.form.get("filename")
        if filename is None:
            flask.abort(400)

        if len(files) == 0:
            flask.abort(400)

        paths = []
        for file in files.values():
            paths.append(controller.save_file(file, filename))
        return paths

    @bp.get("/_files/<path:path>")
    def _get_file(path):
        return flask.send_file(controller.get_file(path))

    @bp.get("/_assets/logo")
    def _logo():
        logo_path = controller.get_workspace().logo_url
        if not logo_path:
            return flask.abort(404)
        return send_from_dist(logo_path, dist_folder=Settings.root_path)

    @bp.get("/favicon.ico")
    def _favicon():
        return _logo()

    @bp.get("/_assets/background")
    def _background():
        background_path = controller.get_workspace().theme

        if not background_path:
            return flask.abort(404)

        return send_from_dist(background_path, dist_folder=Settings.root_path)

    @bp.route("/_hooks/<path:path>", methods=["POST", "GET", "PUT", "DELETE", "PATCH"])
    def hook_runner(path):
        hook = controller.get_hook_by_path(path)

        if not hook:
            flask.abort(404)

        if not hook.file:
            flask.abort(500)

        request_data = RequestData(
            query_params=flask.request.args,
            body=flask.request.get_data(as_text=True),
            headers=flask.request.headers,
            method=flask.request.method,
        )

        execution = HookExecution(
            is_initial=controller.is_initial(hook.id),
            request=request_data,
            stage=hook,
        )

        execution.run()
        workflow_engine.notify_ran(execution)

        body, status, headers = execution.get_response()
        return flask.Response(status=status, headers=headers, response=body)

    @bp.get("/_jobs")
    def list_jobs():
        if flask.request.headers.get("Shared-Token") != SIDECAR_SHARED_TOKEN:
            flask.abort(401)

        jobs = controller.get_jobs()

        # used by sidecar - DO NOT CHANGE CONTRACT
        return [{"id": job.id, "schedule": job.schedule} for job in jobs]

    @bp.post("/_jobs/<path:id>")
    def job_runner(id):
        if flask.request.headers.get("Shared-Token") != SIDECAR_SHARED_TOKEN:
            flask.abort(401)

        job = controller.get_job(id)
        if not job:
            flask.abort(404)

        if not job.file:
            flask.abort(500)

        workflow_engine.run_job(job)
        return {"status": "running"}

    @bp.delete("/_executions/<string:execution_id>")
    def abort_execution(execution_id):
        if flask.request.headers.get("Shared-Token") != SIDECAR_SHARED_TOKEN:
            flask.abort(401)
        controller.abort_execution(execution_id)
        return {"status": "deleted"}

    @bp.get("/")
    def index():
        res = send_from_dist("player.html", "player.html")
        return res

    @bp.get("/<path:filename>")
    def spa(filename: str):
        res = send_from_dist(filename, "player.html")
        return res

    return bp



import flask

from ...usage import usage
from .main import MainController


def get_editor_bp(controller: MainController):
    bp = flask.Blueprint("editor_requirements", __name__)

    @bp.get("/")
    @usage
    def _get_requirements():
        return controller.get_requirements()

    @bp.post("/")
    @usage
    def _create_requirement():
        data = flask.request.json

        if not data:
            flask.abort(400)

        return controller.create_requirement(data["name"], data.get("version"))

    @bp.delete("/<name>")
    @usage
    def _delete_requirement(name: str):
        return controller.delete_requirement(name)

    @bp.get("/recommendations")
    @usage
    def _get_requirements_recommendation():
        return controller.get_requirements_recommendations()

    return bp



import threading, traceback, uuid
from dataclasses import dataclass
from typing import Any, ClassVar, Dict, List, Optional

from ..local_log import LocalLogMessage, append_to_or_create_log_file
from ..modules import import_as_new
from ..monitoring import LogMessage, log
from ..utils.environment import IS_PREVIEW
from ..repositories import StageRunRepository
from ..repositories.stage_run import StageRun
from ..repositories.project.project import ActionStage


class NoExecutionFound(Exception):
    def __str__(self) -> str:
        return "No execution found. If you are running this locally, make sure you are running it with `abstra serve`. For more information, see https://docs.abstra.io"


@dataclass
class RequestData:
    headers: Dict[str, str]
    body: str
    method: str
    query_params: Dict[str, str]


class LockFailedMessage(Exception):
    def __init__(self, status: Optional[str]) -> None:
        super().__init__(status)


class InvalidStageRunId(Exception):
    pass


class UnsetStageRun(Exception):
    pass


class Execution:
    type = "execution"
    abstra_run_key = "abstra-run-id"
    executions: ClassVar[Dict[int, "Execution"]] = {}

    id: str
    is_initial: bool
    status = "waiting"
    stage: ActionStage
    request: RequestData

    stage_run_draft: Optional[StageRun] = None
    stage_run_freezed: Optional[StageRun] = None

    @classmethod
    def get_execution_by_id(cls, execution_id: str) -> Optional["Execution"]:
        for execution in cls.executions.values():
            if execution.id == execution_id:
                return execution
        return None

    @classmethod
    def get_current_execution(cls) -> Optional["Execution"]:
        thread_id = threading.get_ident()
        return cls.executions.get(thread_id)

    def __init__(
        self,
        stage: ActionStage,
        is_initial: bool,
        request: RequestData,
        execution_id=None,
    ):
        self.stderr: List[str] = []
        self.stdout: List[str] = []
        self.context: Dict = {}
        self.request = request
        self.stage = stage
        self.is_initial = is_initial
        self.thread_id = threading.get_ident()
        Execution.executions[self.thread_id] = self
        self.id = execution_id or uuid.uuid4().__str__()

    @property
    def is_preview(self) -> bool:
        return IS_PREVIEW

    def stdio(self, type: str, text: str):
        if type == "stderr":
            self.stderr.append(text)
        elif type == "stdout":
            self.stdout.append(text)
        if self.is_preview:
            append_to_or_create_log_file(
                LocalLogMessage(
                    event=type,
                    payload={"log": text},
                    execution_id=self.id,
                    stage_run_id=self.stage_run.id if self.stage_run else None,
                    runtime_type=self.stage.type_name,
                    runtime_id=self.stage.id,
                )
            )

    def log(self, event: str, payload: Any):
        if event == "heartbeat":
            return
        log(
            LogMessage(
                event=event,
                payload=payload,
                executionId=self.id,
                runtime_type=self.stage.type_name,
                runtime_name=self.stage.title,
            )
        )

    def handle_started(self) -> str:
        return "running"

    def handle_success(self) -> str:
        return "finished"

    def handle_failure(self, e: Exception) -> str:
        traceback.print_exc()
        return "failed"

    def handle_lock_failed(self) -> None:
        stage_run_id = self.stage_run.id if self.stage_run else None
        stage_run_status = self.stage_run.status if self.stage_run else None

        self.log(
            "lock-failed",
            {
                "execution_id": self.id,
                "stage": self.stage.id,
                "stage_run_id": stage_run_id,
                "stage_run_status": stage_run_status,
            },
        )
        raise LockFailedMessage(stage_run_status)

    def setup_context(self, request: RequestData):
        raise NotImplementedError()

    @property
    def stage_run(self) -> Optional[StageRun]:
        return self.stage_run_draft

    @stage_run.setter
    def stage_run(self, stage_run: StageRun) -> None:
        self.stage_run_freezed = stage_run
        self.stage_run_draft = stage_run.clone()

    def get_not_abandoned_stage_run(self, stage_run: StageRun) -> StageRun:
        if stage_run.status != "abandoned":
            return stage_run

        next_matches = StageRunRepository.find(
            {"parent_id": stage_run.id, "stage": self.stage.id}
        )

        assert (
            len(next_matches) == 1
        ), "Internal error: abandoned stage run does not have exactly one child"

        return self.get_not_abandoned_stage_run(next_matches[0])

    def init_stage_run(self, stage_run_id: Optional[str] = None) -> None:
        if stage_run_id:
            stage_run = StageRunRepository.get(stage_run_id)
            self.stage_run = self.get_not_abandoned_stage_run(stage_run)

            if stage_run.stage == self.stage.id:
                self.stage_run = stage_run
                return

            raise InvalidStageRunId()

        if self.is_initial:
            self.stage_run = StageRunRepository.create_initial(stage=self.stage.id)
            return

        raise UnsetStageRun(
            f"This Stage is in the middle of a workflow, but no StageRun was specified"
        )

    def run(self) -> None:
        self.setup_context(self.request)
        self.status = self.handle_started()

        lock_held = self.set_stage_run_status()

        if self.stage_run and not lock_held:
            return self.handle_lock_failed()

        try:
            try:
                import_as_new(self.stage.file)
            except SystemExit as e:
                if e.code is not None and e.code != 0:
                    raise e

            self.status = self.handle_success()
        except Exception as e:
            self.status = self.handle_failure(e)
        finally:
            self.unbind()
            self.set_stage_run_status()

    def set_stage_run_status(self) -> bool:
        if not self.stage_run:
            return False

        return StageRunRepository.change_state(self.stage_run.id, self.status, self.id)

    def unbind(self) -> None:
        del Execution.executions[self.thread_id]


def get_current_execution_throwable() -> Execution:
    execution = Execution.get_current_execution()
    if not execution:
        raise NoExecutionFound()
    return execution



from contextlib import suppress
from dataclasses import dataclass, field
from datetime import datetime
import json
from pathlib import Path
from typing import List, Optional, Dict

from abstra_internals.utils.environment import IS_PREVIEW
from abstra_internals.utils.file import LOCAL_LOG_FOLDER

from .utils.validate import validate_json


@dataclass
class LocalLogMessage:
    execution_id: str
    payload: Dict[str, str]
    event: Optional[str] = ""
    stage_run_id: Optional[str] = ""
    runtime_type: Optional[str] = ""
    runtime_id: Optional[str] = ""
    created_at: float = field(default_factory=lambda: datetime.now().timestamp())

    def json(self):
        return dict(
            event=str(self.event),
            executionId=str(self.execution_id),
            stageRunId=str(self.stage_run_id),
            payload=dict(self.payload),
            runtimeType=str(self.runtime_type),
            runtimeId=str(self.runtime_id),
            createdAt=float(self.created_at),
        )

    @classmethod
    def from_json(cls, json):
        validated_json = _validate_log_message_json(json)

        return LocalLogMessage(
            event=validated_json["event"],
            execution_id=validated_json["executionId"],
            stage_run_id=validated_json["stageRunId"],
            payload=validated_json["payload"],
            runtime_type=validated_json["runtimeType"],
            runtime_id=validated_json["runtimeId"],
            created_at=validated_json["createdAt"],
        )


def _validate_log_message_json(json):
    expected_types = {
        "event": str,
        "executionId": str,
        "stageRunId": Optional[str],
        "payload": dict,
        "runtimeType": str,
        "runtimeId": str,
        "createdAt": float,
    }
    return validate_json(json, expected_types)


def append_to_or_create_log_file(message: LocalLogMessage) -> None:
    if not IS_PREVIEW:
        return

    execution_id = message.execution_id
    if not execution_id:
        return

    log_file = Path(LOCAL_LOG_FOLDER) / f"{execution_id}.log"

    if not log_file.parent.exists():
        log_file.parent.mkdir(parents=True)
    with open(log_file, "a", encoding="utf-8") as f:
        f.write(json.dumps(message.json()) + "\n")


def get_local_logs(execution_id: Optional[str]) -> List[LocalLogMessage]:
    if not IS_PREVIEW:
        return []

    log_file = Path(LOCAL_LOG_FOLDER) / f"{execution_id}.log"

    if not log_file.exists():
        return []

    try:
        with open(log_file, "r", encoding="utf-8") as f:
            logs: List[LocalLogMessage] = []
            for line in f.readlines():
                with suppress(Exception):
                    logs.append(LocalLogMessage.from_json(json.loads(line)))
            return logs
    except Exception:
        print("Error reading log file")
        return []



from dataclasses import dataclass
import jwt, datetime, typing
from .utils import is_valid_email
from .utils.environment import PUBLIC_KEY


def decode_jwt(jwt_str: str):
    try:
        if PUBLIC_KEY:
            return jwt.decode(jwt_str, key=PUBLIC_KEY, algorithms=["RS256"])
        return jwt.decode(jwt_str, options={"verify_signature": False})
    except Exception as e:
        print("error decoding jwt", e)
        return None


def endcode_fake_jwt(email: str):
    return jwt.encode(
        key="fake",
        algorithm="HS256",
        payload={
            "email": email,
            "exp": datetime.datetime.utcnow() + datetime.timedelta(days=7),
        },
    )


@dataclass
class UserClaims:
    """The response from the authentication process

    Attributes:
      email (str): The email address of the user
      claims (dict): The claims from the JWT token
    """

    claims: typing.Dict[str, typing.Any]

    @property
    def email(self) -> str:
        return self.claims["email"]

    @classmethod
    def from_jwt(cls, jwt_str: str) -> typing.Optional["UserClaims"]:
        claims = decode_jwt(jwt_str)
        if claims is None:
            return None

        email = claims.get("email")
        if not is_valid_email(email):
            return None

        return cls(claims)

    def __getattr__(self, name):
        return self.claims.get(name)
